@@ -28,6 +28,11 @@ def die(msg, error_code=1):
   sys.exit(error_code)
 
 
+def log(msg, v=False):
+  if v:
+    print(msg, file=sys.stderr)
+
+
 def parse_bool(option, opt_str, _, parser):
   setattr(parser.values, option.dest, not opt_str.startswith('--no'))
 
@@ -49,25 +54,61 @@ def configure_clp():
       help='Whether to use pypi to resolve dependencies; Default: use pypi')
 
   parser.add_option(
-      '--cache-dir',
-      dest='cache_dir',
-      default=os.path.expanduser('~/.pex/install'),
-      help='The local cache directory to use for speeding up requirement '
-           'lookups; Default: ~/.pex/install')
+      '--zip-safe', '--not-zip-safe',
+      dest='zip_safe',
+      default=True,
+      action='callback',
+      callback=parse_bool,
+      help='Whether or not the sources in the pex file are zip safe.  If they are '
+           'not zip safe, they will be written to disk prior to execution; '
+           'Default: zip safe.')
 
   parser.add_option(
-      '-p', '--pex-name',
+      '--always-write-cache',
+      dest='always_write_cache',
+      default=False,
+      action='store_true',
+      help='Always write the internally cached eggs to disk prior to invoking '
+           'the pex source code.  This can use less memory in RAM constrained '
+           'environments. [Default: %default]')
+
+  parser.add_option(
+      '--ignore-errors',
+      dest='ignore_errors',
+      default=False,
+      action='store_true',
+      help='Ignore run-time requirement resolution errors when invoking the pex. '
+           '[Default: %default]')
+
+  parser.add_option(
+      '--inherit-path',
+      dest='inherit_path',
+      default=False,
+      action='store_true',
+      help='Inherit the contents of sys.path (including site-packages) running the pex. '
+           '[Default: %default]')
+
+  parser.add_option(
+      '--cache-dir',
+      dest='cache_dir',
+      default=os.path.expanduser('~/.pex/build'),
+      help='The local cache directory to use for speeding up requirement '
+           'lookups; [Default: %default]')
+
+  parser.add_option(
+      '-o', '-p', '--output-file', '--pex-name',
       dest='pex_name',
       default=None,
       help='The name of the generated .pex file: Omiting this will run PEX '
-           'immediately and not save it to a file')
+           'immediately and not save it to a file.')
 
   parser.add_option(
       '-e', '--entry-point',
       dest='entry_point',
       default=None,
       help='The entry point for this pex; Omiting this will enter the python '
-           'REPL with sources and requirements available for import')
+           'REPL with sources and requirements available for import.  Can be '
+           'either a module or EntryPoint (module:function) format.')
 
   parser.add_option(
       '-r', '--requirement',
@@ -107,6 +148,13 @@ def configure_clp():
 def build_pex(args, options):
   pex_builder = PEXBuilder(path=safe_mkdtemp())
 
+  pex_info = pex_builder.info
+
+  pex_info.zip_safe = options.zip_safe
+  pex_info.always_write_cache = options.always_write_cache
+  pex_info.ignore_errors = options.ignore_errors
+  pex_info.inherit_path = options.inherit_path
+
   fetchers = [Fetcher(options.repos)]
 
   if options.pypi:
@@ -114,20 +162,16 @@ def build_pex(args, options):
 
   resolver = Resolver(cache=options.cache_dir, fetchers=fetchers, install_cache=options.cache_dir)
 
-  if options.requirements:
-    print('Resolving requirements:')
-    for req in options.requirements:
-      print('  - %s' % req)
-
   resolveds = resolver.resolve(options.requirements)
 
+  if resolveds:
+    log('Resolved distributions:', v=options.verbosity)
   for pkg in resolveds:
-    print('Resolved distribution: %s [%s]' % (pkg, pkg.location))
+    log('  %s' % pkg, v=options.verbosity)
     pex_builder.add_distribution(pkg)
     pex_builder.add_requirement(pkg.as_requirement())
 
   for source_dir in options.source_dirs:
-    print('Distilling %s into egg...' % source_dir, end='\r')
     dist = Installer(source_dir).distribution()
     if not dist:
       die('Failed to run installer for %s' % source_dir, CANNOT_DISTILL)
@@ -135,23 +179,22 @@ def build_pex(args, options):
     if not egg_path:
       die('Failed to distill %s into egg' % dist, CANNOT_DISTILL)
     pex_builder.add_egg(egg_path)
-    print('Successfully distilled %s into %s' % (source_dir, egg_path))
 
   if options.entry_point is not None:
-    print('Setting entry point to %s' % options.entry_point)
+    log('Setting entry point to %s' % options.entry_point, v=options.verbosity)
     pex_builder.info.entry_point = options.entry_point
   else:
-    print('Creating environment PEX.')
+    log('Creating environment PEX.', v=options.verbosity)
 
   if options.pex_name is not None:
-    print('Saving PEX file to %s' % options.pex_name)
+    log('Saving PEX file to %s' % options.pex_name, v=options.verbosity)
     tmp_name = options.pex_name + '~'
     safe_delete(tmp_name)
     pex_builder.build(tmp_name)
     os.rename(tmp_name, options.pex_name)
   else:
     pex_builder.freeze()
-    print('Running PEX file at %s with args %s' % (pex_builder.path(), args))
+    log('Running PEX file at %s with args %s' % (pex_builder.path(), args), v=options.verbosity)
     pex = PEX(pex_builder.path())
     return pex.run(args=list(args))
 
