@@ -16,7 +16,7 @@ import sys
 
 from .tracer import Tracer
 
-from pkg_resources import find_distributions, Distribution, Requirement
+from pkg_resources import Distribution, find_distributions, Requirement
 
 TRACER = Tracer(predicate=Tracer.env_filter('PEX_VERBOSE'),
     prefix='twitter.common.python.interpreter: ')
@@ -61,13 +61,19 @@ class PythonIdentity(object):
   class InvalidError(Error): pass
   class UnknownRequirement(Error): pass
 
-  @staticmethod
-  def get():
+  HASHBANGS = {
+    'CPython': 'python%(major)d.%(minor)d',
+    'Jython': 'jython',
+    'PyPy': 'pypy',
+  }
+
+  @classmethod
+  def get(cls):
     if hasattr(sys, 'subversion'):
       subversion = sys.subversion[0]
     else:
       subversion = 'CPython'
-    return PythonIdentity(subversion, sys.version_info[0], sys.version_info[1], sys.version_info[2])
+    return cls(subversion, sys.version_info[0], sys.version_info[1], sys.version_info[2])
 
   @classmethod
   def from_id_string(cls, id_string):
@@ -129,7 +135,12 @@ class PythonIdentity(object):
     return self.distribution in requirement
 
   def hashbang(self):
-    return '#!/usr/bin/env python%s.%s' % self._version[0:2]
+    hashbang_string = self.HASHBANGS.get(self.interpreter, 'CPython') % {
+      'major': self._version[0],
+      'minor': self._version[1],
+      'patch': self._version[2],
+    }
+    return '#!/usr/bin/env %s' % hashbang_string
 
   def __str__(self):
     return '%s-%s.%s.%s' % (self._interpreter,
@@ -166,17 +177,20 @@ class PythonInterpreter(object):
     COMPATIBLE_SETUPTOOLS = Requirement.parse('setuptools>=1.0', replacement=False)
   except TypeError:
     COMPATIBLE_SETUPTOOLS = Requirement.parse('setuptools>=1.0')
-
+  
   class Error(Exception): pass
   class IdentificationError(Error): pass
+  class InterpreterNotFound(Error): pass
 
   @classmethod
   def get(cls):
     return cls(sys.executable, interpreter=PythonIdentity.get())
 
   @classmethod
-  def all(cls, paths=os.getenv('PATH').split(':')):
-    return cls.filter(PythonInterpreter.find(paths))
+  def all(cls, paths=None):
+    if paths is None:
+      paths = os.getenv('PATH', '').split(':')
+    return cls.filter(cls.find(paths))
 
   @classmethod
   def from_binary(cls, binary):
@@ -278,7 +292,7 @@ class PythonInterpreter(object):
       self._identity = interpreter or PythonIdentity.get()
       self._distribute = distribute_path or self._find_distribute()
     else:
-      self._identity = interpreter or PythonInterpreter.from_binary(self._binary).identity
+      self._identity = interpreter or self.from_binary(self._binary).identity
       self._distribute = distribute_path
 
   def _find_distribute(self):
